@@ -35,6 +35,12 @@ setup_S_model <- function(formula, formLong, dataSurv, LSurvdat, timeVar, assoc,
   YS_FE_elements <- gsub("\\s", "", strsplit(strsplit(as.character(FE_formS), split="~")[[3]], split=c("\\+"))[[1]])
   FML <- ifelse(strsplit(as.character(FE_formS), split="~")[[3]]=="-1", 1, strsplit(as.character(FE_formS), split="~")[[3]])
   YSform2 <- formula(paste(" ~ ", FML))
+  # if(length(which(sapply(dataSurv, class)=="factor"))>0){ # deal with factors when modalities are missing
+  #   factors_columns <- which(sapply(dataSurv, class)=="factor")
+  #   for(fctc in factors_columns){
+  #     if(length(unique(dataSurv[, fctc]))< length(levels(dataSurv[, fctc]))) dataSurv[, fctc] <- as.integer(dataSurv[, fctc])-1
+  #   }
+  # }
   DFS <- model.matrix(YSform2, model.frame(YSform2, dataSurv, na.action=na.pass))
   if(colnames(DFS)[1]=="(Intercept)") colnames(DFS)[1] <- "Intercept"
   if(grepl("inla.surv", YS)){
@@ -195,12 +201,12 @@ setup_Y_model <- function(formula, dataset, family, k){
 #' @return FE values of the fixed effects
 setup_FE_model <- function(formula, dataset, timeVar, k, dataOnly){
   FE_form <- nobars(formula)
-  if(length(which(sapply(dataset, class)=="factor"))>0 & !dataOnly){ # deal with factors when modalities are missing
-    factors_columns <- which(sapply(dataset, class)=="factor")
-    for(fctc in factors_columns){
-      if(length(unique(dataset[, fctc]))< length(levels(dataset[, fctc]))) dataset[, fctc] <- as.integer(dataset[, fctc])-1
-    }
-  }
+  # if(length(which(sapply(dataset, class)=="factor"))>0 & !dataOnly){ # deal with factors when modalities are missing
+  #   factors_columns <- which(sapply(dataset, class)=="factor")
+  #   for(fctc in factors_columns){
+  #     if(length(unique(dataset[, fctc]))< length(levels(dataset[, fctc]))) dataset[, fctc] <- as.integer(dataset[, fctc])-1
+  #   }
+  # }
   FE <- model.matrix(FE_form, model.frame(FE_form, dataset, na.action=na.pass))
   #if(colnames(FE)[1]=="(Intercept)") colnames(FE)[1] <- "Intercept"
   colnames(FE) <- gsub(":", ".X.", gsub("\\s", ".", colnames(FE)))
@@ -234,5 +240,134 @@ setup_RE_model <- function(formula, dataset, k){
   return(list(colnames(RE_mat), RE_mat))
 }
 
+#' Setup scopy
+#' @description Setup weights for non-linear effects
+#' input:
+#' @param n number of knots
+#' output:
+#' @return Matrix with weights for scopy
+INLAjoint.scopy.define <- function(n = 5L)
+{
+  stopifnot(n == 2 || n >= 5)
+  if (n == 2) {
+    VV = cbind(1, c(-0.5, 0.5))
+  }
+  else {
+    Q <- INLAjoint.rw2(n, scale.model = TRUE)
+    e <- eigen(Q)
+    idx.remove <- (n - 1):n
+    V <- e$vectors[, -idx.remove, drop = FALSE]
+    lambda <- e$values[-idx.remove]
+    VV <- matrix(0, n, n)
+    VV[, 1:2] <- cbind(rep(1, n), seq(-0.5, 0.5, len = n))
+    m <- n - 2
+    for (i in seq_len(m)) {
+      j <- m - i + 3
+      VV[, j] <- V[, i] * sqrt(1/lambda[i])
+    }
+  }
+  return(list(n = n, W = VV))
+}
+
+#' Setup rw2
+#' @description Setup random walk of order 2
+#' input:
+#' @param n number of knots
+#' @param order order 2
+#' @param ... additional arguments passed to INLAjoint.rw
+#' output:
+#' @return random walk 2
+INLAjoint.rw2 <- function (n, order=2L, ...)
+{
+  INLAjoint.rw(n, order = order, ...)
+}
+
+#' Setup rw
+#' @description Setup random walk
+#' input:
+#' @param n number of knots
+#' @param order order 1
+#' @param sparse boolean, sparsity
+#' @param scale.model boolean, scale
+#' @param cyclic boolean, cyclic
+#' output:
+#' @return random walk 1
+INLAjoint.rw <- function (n, order = 1L, sparse = TRUE, scale.model = FALSE,
+          cyclic = FALSE)
+{
+  stopifnot(n >= 1L + 2L * order)
+  if (scale.model) {
+    if (!cyclic) {
+      rd <- order
+    }
+    else {
+      if (order == 1L) {
+        rd <- 1L
+      }
+      else {
+        rd <- order - 1L
+      }
+    }
+    Q <- INLAjoint.rw(n, order = order, sparse = sparse, scale.model = FALSE,
+                 cyclic = cyclic)
+    fac <- exp(mean(log(diag(INLAjoint.ginv(as.matrix(Q), rankdef = rd)))))
+    Q <- fac * Q
+  }
+  else {
+    if (!cyclic) {
+      U <- diff(diag(n), diff = order)
+      Q <- t(U) %*% U
+    }
+    else {
+      m <- 2L * order + 1L
+      k <- 1L + order
+      U <- diff(diag(m), diff = order)
+      U <- t(U) %*% U
+      Q <- toeplitz(c(U[k, k:m], rep(0, n - m), U[k, m:(k +
+                                                          1L)]))
+    }
+  }
+  return(if (sparse) INLA::inla.as.sparse(Q) else Q)
+}
+
+#' Setup ginv
+#' @description Setup random walk
+#' input:
+#' @param x input
+#' @param tol tolerance
+#' @param rankdef rank
+#' output:
+#' @return ginv
+INLAjoint.ginv <- function (x, tol = sqrt(.Machine$double.eps), rankdef = NULL)
+{
+  if (!is.matrix(x)) {
+    x <- as.matrix(x)
+  }
+  if (length(dim(x)) > 2L || !(is.numeric(x) || is.complex(x))) {
+    stop("'x' must be a numeric or complex matrix")
+  }
+  xsvd <- svd(x)
+  if (is.complex(x)) {
+    xsvd$u <- Conj(xsvd$u)
+  }
+  if (is.null(rankdef) || rankdef == 0L) {
+    Positive <- xsvd$d > max(tol * xsvd$d[1L], 0)
+  }
+  else {
+    n <- length(xsvd$d)
+    stopifnot(rankdef >= 1L && rankdef <= n)
+    Positive <- c(rep(TRUE, n - rankdef), rep(FALSE, rankdef))
+  }
+  if (all(Positive)) {
+    xsvd$v %*% (1/xsvd$d * t(xsvd$u))
+  }
+  else if (!any(Positive)) {
+    array(0, dim(x)[2L:1L])
+  }
+  else {
+    xsvd$v[, Positive, drop = FALSE] %*% ((1/xsvd$d[Positive]) *
+                                            t(xsvd$u[, Positive, drop = FALSE]))
+  }
+}
 
 
